package foraging;

import com.cyberbotics.webots.controller.GPS;
import com.cyberbotics.webots.controller.Robot;


public class Recharge extends foraging_stub.Recharge_stub {
    
    private final Robot robot;
    private final GPS gps;
    private final FSM fsm;
    private float battery;
    
    
    public Recharge(Robot robot, GPS gps) {
        super();
        this.robot = robot;
        this.gps = gps;
        fsm = new FSM(IDLE, RECHARGE);
    }
    
    
    @Override
    public void init() {
        fsm.init();
    }
    
    
    @Override
    public void step() {
        // read sensor data
        battery = (float)robot.batterySensorGetValue();
        //System.out.println("Recharge: battery = " + battery);
        fsm.exec();
    }
    
    
    private final FSM.State IDLE = new FSM.State() {
        @Override
        protected void entry() {
            System.out.println("Recharge: idle");
        }
        
        @Override
        protected void exec() {
            double[] pos = gps.getValues();
            float x = (float)pos[0];
            float z = (float)pos[2];
        	if (battery < Math.max(30, 50*Misc.dist(x, z, Misc.CHARGER.x, Misc.CHARGER.z)/1.1f)) {
                transition(RECHARGE);
            }
        }
    };
    
    
    private final FSM.State RECHARGE = new FSM.State() {
    	@Override
        protected void entry() {
            System.out.println("Recharge: recharge");
            _homing.send(Misc.CHARGER);
        }
    	
    	@Override
        protected void exec() {
            if (battery > 99.5f) {  // resume normal behavior
                transition(IDLE);
            } else {  // returning to charger or recharging
                _homing.send(Misc.CHARGER);
            }
        }
    };
    
}
